/* ### CHECK-ME-BEFORE-RELEASE ### */
/*
 * Title:   piendian.c
 * Created: Sun Jan 30 22:03:55 2000
 * Authors: Tim Weyrich
 *     $Id: piendian.c,v 1.2 2003/11/19 10:37:30 weyrich Exp $
 *
 * Copyright (c) 2000--2003
 * Gilbert Baumann <unk6@stud.uni-karlsruhe.de>
 * Markus Noga <markus@noga.de>
 * Tim Weyrich <weyrich@inf.ethz.ch>
 *
 * This file is part of the Pointshop3D system.
 * See http: *www.pointshop3d.com/ for more information.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 * Contact info@pointshop3d.com if any conditions of this
 * licensing are not clear to you.
 *
 * ---------------------------------------------------------------
 *
 * $Log: piendian.c,v $
 * Revision 1.2  2003/11/19 10:37:30  weyrich
 * added Pointshop3D headers
 *
 * Revision 1.1  2001/11/20 16:36:50  weyrich
 * new files: piendian.[ch]
 *
 *
 */

#include "piendian.h"

/*****************************************************************************/

uint16  *pi_swap_uint16(uint16 *ui)
{
  FAKE_PIENDIAN_USAGE;
  *ui = (*ui>>8) | (*ui<<8);
  return ui;
}

uint32  *pi_swap_uint32(uint32 *ui)
{
  *ui = (*ui>>24) | ((*ui>>8)&0xff00) | ((*ui<<8)&0x00ff0000) | (*ui<<24);
  return ui;
}

#ifndef NO_INT64
uint64  *pi_swap_uint64(uint64 *ui)
{
#  if defined(WIN32) || defined(__STRICT_ANSI__)
  uint32  *a=(uint32*) ui, h;
  pi_swap_uint32(&a[0]);
  pi_swap_uint32(&a[1]);
  h=a[0]; a[0]=a[1]; a[1]=h;
  return (uint64 *)a;
#  else
  *ui = ((*ui>>56) |
         ((*ui>>40)&0x0000ff00) |
         ((*ui>>24)&0x00ff0000) |
         ((*ui>>8)&0xff000000) |
         ((*ui<<8)&0x000000ff00000000ULL) |
         ((*ui<<24)&0x0000ff0000000000ULL) |
         ((*ui<<40)&0x00ff000000000000ULL) |
         (*ui<<56));
  return ui;
#  endif
}
#endif

/*****************************************************************************/

#if !( defined(WIN32) || defined(__STRICT_ANSI__) )
int16  *pi_swap_int16(int16 *i)    { return ((int16 *)pi_swap_uint16(((uint16 *)(i)))); }
int32  *pi_swap_int32(int32 *i)    { return ((int32 *)pi_swap_uint32(((uint32 *)(i)))); }
#  ifndef NO_INT64
sint64  *pi_swap_int64(sint64 *i)  { return ((sint64 *)pi_swap_uint64(((uint64 *)(i)))); }
#  endif

float  *pi_swap_float(float *f)    { return ((float *)pi_swap_uint32(((uint32 *)(f)))); }
#  ifndef NO_INT64
double  *pi_swap_double(double *d) { return ((double *)pi_swap_uint64(((uint64 *)(d)))); }
#  else
double  *pi_swap_double(double *d)
{
  uint32  *a=(uint32*)d, h;
  pi_swap_uint32(&a[0]);
  pi_swap_uint32(&a[1]);
  h=a[0]; a[0]=a[1]; a[1]=h;
  return (double*)a;
}
#  endif

/* And finally, to make code generation easier: ******************************/

char   *pi_swap_char(char *c)   { return c; }
uchar  *pi_swap_uchar(uchar *c) { return c; }
int8   *pi_swap_int8(int8 *i)   { return i; }
uint8  *pi_swap_uint8(uint8 *i) { return i; }

#endif

/*****************************************************************************/
