/* ### CHECK-ME-BEFORE-RELEASE ### */
/*
 * Title:   xthread.c
 * Created: Spring 2000
 * Authors: Markus Noga
 *     $Id: xthread.c,v 1.2 2003/11/19 10:37:31 weyrich Exp $
 *
 * Copyright (c) 2000--2003
 * Gilbert Baumann <unk6@stud.uni-karlsruhe.de>
 * Markus Noga <markus@noga.de>
 * Tim Weyrich <weyrich@inf.ethz.ch>
 *
 * This file is part of the Pointshop3D system.
 * See http: *www.pointshop3d.com/ for more information.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 * Contact info@pointshop3d.com if any conditions of this
 * licensing are not clear to you.
 *
 * ---------------------------------------------------------------
 *
 * $Log: xthread.c,v $
 * Revision 1.2  2003/11/19 10:37:31  weyrich
 * added Pointshop3D headers
 *
 *
 */

/*! \file   xthread_c
    \brief  UNIX/X11 implementation of thread functions.
    \author Markus L. Noga
*/

#include <pithread.h>
#include <pisystem.h>

/*! \brief Helper structure containing thread startup data
 */
typedef struct {
  void* (*func)(void*);
  void *arg;
} pithread_startup_t;


/*! \brief Helper function to start threads.
    We'd like for threads to operate in asynchronous cancellation
    mode - i.e., cancel means cancel *now*. This can only be set
    from within the thread itself, which is what this function does.
*/
static void *xthread_helper(void *param) {
  pithread_startup_t *data=(pithread_startup_t*) param;
  void* (*func)(void*)=data->func;
  void*   arg         =data->arg;
  
  free(data);
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
  
  return func(arg);
}

int pithread_create(pithread_t *__thread,
		    const pithread_attr_t *__attr,
		    void *(*__start_routine) (void *),
		    void *__arg) {
  pithread_startup_t *data=(pithread_startup_t *) malloc(sizeof(pithread_startup_t));
  
  if(!data)
    return -1;
  
  data->func=__start_routine;
  data->arg =__arg;
  
  return pthread_create(__thread,__attr, xthread_helper, data);
}
