// ### CHECK-ME-BEFORE-RELEASE ###
//
// Title:   sflmat.cpp
// Created: Thu May 16 18:42:18 2002
// Authors: Tim Weyrich <weyrich@inf.ethz.ch>
//
// Copyright (c) 2001--2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//
// ---------------------------------------------------------------
//
// $Log: sflmat.cpp,v $
// Revision 1.4  2003/11/19 13:04:03  weyrich
// added Pointshop3D headers
//
// Revision 1.3  2003/03/20 14:52:52  weyrich
// smaller fixes of implicit coordinate system transformations
//
// Revision 1.2  2002/05/23 13:59:56  weyrich
// First official release.
//
// Revision 1.1  2002/05/17 13:58:16  weyrich
// PointShop3D integration nearly finished
//

#include <pisystem.h>
#include <pistring.h>
#include <pimath.h>

#include "sflmat.h"

using namespace sflMat;

#define  DOUBLE_EPS  1e-8

///////////////////////////////////////////////////////////////////////////////

void  sflMat::setIdentity16(float mat[16])
{
  int  i, j;
  for (i=0; i<4; i++)
    for (j=0; j<4; j++)
      mat[4*i + j] = (float)(i==j);
}

void  sflMat::setIdentity16(double mat[16])
{
  int  i, j;
  for (i=0; i<4; i++)
    for (j=0; j<4; j++)
      mat[4*i + j] = (double)(i==j);
}

void  sflMat::setIdentity32(float mat[32])
{
  setIdentity16(mat);
  setIdentity16(mat+16);
}

void  sflMat::setIdentity32(double mat[32])
{
  setIdentity16(mat);
  setIdentity16(mat+16);
}

void  sflMat::setZero16(double mat[16])
{
  for (int i=0; i<16; i++)
    mat[i] = 0.0;
}

#if 0
bool  sflMat::invMat16(double dest[16], const double mat[16])
{/* matrix inversion by gaussian elimination, using total pivotization */

  const int  n = 4;

  int       m = 2*n;
  int       i, j, k;
  int      *orirow = (int*)alloca(n*sizeof(int));
  int      *oricolumn = (int*)alloca(n*sizeof(int));
  double   *a = (double *)alloca(n*m * sizeof(double));
  double    f;

  for (i=0; i<n; i++)
    {
      for(j=0; j<n; j++)
        a[(i)*m + j] = mat[(i)*n + j];
      for(j=n; j<m; j++)
        a[(i)*m + j] = 0.0;
      a[(i)*m + n+i] = 1.0;
    }

  for (i=0; i<n; i++)
    orirow[i] = i;

  for (i=0; i<n; i++)
    oricolumn[i] = i;

  for (j=0; j<n; j++)
    {
      /* Pivotelement suchen */
      {
        double  max = 0.0;
        int     pi, pj, pivi=0, pivj=0;
        for (pi=j; pi<n; pi++)
          for (pj=j; pj<n; pj++)
            if (fabs(a[(pi)*m + pj]) > max)
              {
                max = fabs(a[(pi)*m + pj]);
                pivi = pi;
                pivj = pj;
              }
        if (max <= 1e-10)
          {
            fprintf(stderr, "(%d: %g)\n", j, a[j*m + j]);
            return false;
          }

        //pivi = i;
        //pivj = j;

        if (pivi != j)
          {
            double  temp;

            /* Pivotzeile ausrichten */
            i = orirow [pivi]; orirow [pivi] = orirow [j]; orirow [j] = i;
            for (i=j; i<n; i++)
              { temp = a[(pivi)*m + i]; a[(pivi)*m + i] = a[(j)*m + i]; a[(j)*m + i] = temp; }
            for (i=0; i<n; i++)
              { temp = a[(i)*m + n+pivi]; a[(i)*m + n+pivi] = a[(i)*m + n+j]; a[(i)*m + n+j] = temp; }
          }

        if (pivj != j)
          {
            double  temp;

            /* Pivotspalte ausrichten */
            i = oricolumn [pivj]; oricolumn [pivj] = oricolumn [j]; oricolumn [j] = i;
            for (i=0; i<n; i++)
              {
                temp = a[(i)*m + pivj]; a[(i)*m + pivj] = a[(i)*m + j]; a[(i)*m + j] = temp;
                temp = a[(pivj)*m + n+i]; a[(pivj)*m + n+i] = a[(j)*m + n+i]; a[(j)*m + n+i] = temp;
              }
          }
      }

      /* Zeilen aufaddieren */
      for (i=0; i<j; i++)
        if (a[(i)*m + j])
          {
            f = - (a[(i)*m + j] / a[(j)*m + j]);
            for (k=j; k<m; k++)
              a[(i)*m + k] += a[(j)*m + k] * f;
          }
      for (i=j+1; i<n; i++)
        if (a[(i)*m + j])
          {
            f = - (a[(i)*m + j] / a[(j)*m + j]);
            for (k=j; k<m; k++)
              a[(i)*m + k] += a[(j)*m + k] * f;
          }
    }

  /* Inverse auslesen */
  for (i=0; i<n; i++)
    for (j=0; j<n; j++)
      dest[(orirow[i])*n + oricolumn[j]] = a[(i)*m + n+j] / a[(i)*m + i];

  return true;
}
#else
//-----------------------------------------------------------------------------
// Matrix inverse by Gauss-Jordan elimination
//-----------------------------------------------------------------------------

static void  gjScaleRow(double mat[16], int row, double scale)
{
  for(int i=0; i<4; i++)
    mat[4*row + i] *= scale;
}

static void  gjSwapRows(double mat[16], int row1, int row2)
{
  for (int i=0; i<4; i++)
    {
      double temp = mat[4*row1 + i]; mat[4*row1 + i]=mat[4*row2 + i]; mat[4*row2 + i] = temp;
    }
}

static void  gjAddToRow(double mat[16], int rowToAdd, double scale, int rowToChange)
{
  for (int i=0; i<4; i++)
    mat[4*rowToChange + i] += mat[4*rowToAdd + i]*scale;
}

bool  sflMat::invMat16(double dest[16], const double mat[16])
{
  double temp[16];
  int i, j;
  double fac;
  double pivot, pivot2;
  int pivrow;

  cpyMat16(temp, mat);

  setIdentity16(dest);

  for (j=0; j<4; j++)
    {
      // Find largest pivot
      pivrow = j;
      pivot = fabs(temp[4*j + j]);
      for (i=j+1; i<4; i++)
        {
          pivot2 = fabs(temp[4*i + j]);
          if (pivot2 > pivot)
            {
              pivot = pivot2;
              pivrow = i;
            }
        }

      // Check for singular matrix
      if (fabs(pivot) < 1e-10)
        {
          setZero16(dest);
          return false;
        }

      // Get pivot into jth row
      if (pivrow != j)
        {
          gjSwapRows(temp, pivrow, j);
          gjSwapRows(dest, pivrow, j);
        }

      // Turn the pivot into 1
      fac = 1.0/temp[4*j + j];
      gjScaleRow(temp, j, fac);
      gjScaleRow(dest, j, fac);

      // Zero out this element in other rows
      for (i=0; i<4; i++)
        {
          if (j != i && temp[4*i + j] != 0.0)
            {
              fac = -(temp[4*i + j]);
              gjAddToRow(temp, j, fac, i);
              gjAddToRow(dest, j, fac, i);
            }
        }
    }
  return true;
}
#endif

void  sflMat::trpMat16(float mat[16])
{
  float  temp;
  int    i, j;
  for (i=0; i<3; i++)
    for (j=i+1; j<4; j++)
      {
        temp = mat[4*i + j]; mat[4*i + j] = mat[4*j + i]; mat[4*j + i] = temp;
      }
}

void  sflMat::trpMat16(double mat[16])
{
  double  temp;
  int     i, j;
  for (i=0; i<3; i++)
    for (j=i+1; j<4; j++)
      {
        temp = mat[4*i + j]; mat[4*i + j] = mat[4*j + i]; mat[4*j + i] = temp;
      }
}

void  sflMat::trpMat16(float dest[16], const float mat[16])
{
  int  i, j;
  for (i=0; i<3; i++)
    for (j=i+1; j<4; j++)
      dest[4*i + j] = mat[4*j + i];
}

void  sflMat::trpMat16(double dest[16], const double mat[16])
{
  int  i, j;
  for (i=0; i<3; i++)
    for (j=i+1; j<4; j++)
      dest[4*i + j] = mat[4*j + i];
}

bool  sflMat::updateInverse(double mat[32])
{
  return invMat16(mat+16, mat);
}

bool  sflMat::isIdentity(double mat[16])
{
  for (int i=0; i<4; i++)
    for (int j=0; j<4; j++)
      if (fabs(mat[4*i + j] - (double)(i==j)) > DOUBLE_EPS)
        return false;
  return true;
}

bool  sflMat::is3x3(double mat[16])
{
  return (is3x4(mat) &&
          fabs(mat[ 3]) <= 1e-10 &&
          fabs(mat[ 7]) <= 1e-10 &&
          fabs(mat[11]) <= 1e-10);
}

bool  sflMat::is3x4(double mat[16])
{
  return (fabs(mat[12]      ) <= 1e-10 &&
          fabs(mat[13]      ) <= 1e-10 &&
          fabs(mat[14]      ) <= 1e-10 &&
          fabs(mat[15] - 1.0) <= 1e-10);
}

#define  APPLY(T,x)                                     \
do                                                      \
{                                                       \
  double  a, b, c;                                      \
  a = ((x)[0] * (T)[0] +                                \
       (x)[1] * (T)[1] +                                \
       (x)[2] * (T)[2] + (T)[3]);                       \
  b = ((x)[0] * (T)[4] +                                \
       (x)[1] * (T)[5] +                                \
       (x)[2] * (T)[6] + (T)[7]);                       \
  c = ((x)[0] * (T)[8] +                                \
       (x)[1] * (T)[9] +                                \
       (x)[2] * (T)[10] + (T)[11]);                     \
  double f = 1.0 / ((x)[0] * (T)[12] +                  \
                    (x)[1] * (T)[13] +                  \
                    (x)[2] * (T)[14] + (T)[15]);        \
  (x)[0] = a*f;                                         \
  (x)[1] = b*f;                                         \
  (x)[2] = c*f;                                         \
} while(0)

bool  sflMat::hasNegDet(double mat[16])
{
  double  o[3] = {0,0,0};
  double  p1[3] = {1,0,0};
  double  p2[3] = {0,1,0};
  double  p3[3] = {0,0,1};
  APPLY(mat, o);
  APPLY(mat, p1);
  APPLY(mat, p2);
  APPLY(mat, p3);
  p1[0] -= o[0];
  p1[1] -= o[1];
  p1[2] -= o[2];
  p2[0] -= o[0];
  p2[1] -= o[1];
  p2[2] -= o[2];
  p3[0] -= o[0];
  p3[1] -= o[1];
  p3[2] -= o[2];

  double  cprod[3] = {p1[1]*p2[2] - p1[2]*p2[1],
                      p1[2]*p2[0] - p1[0]*p2[2],
                      p1[0]*p2[1] - p1[1]*p2[0]};

  return cprod[0]*p3[0] + cprod[1]*p3[1] + cprod[2]*p3[2] < 0;
}

void sflMat::mulMat16Mat16(float dest[16], const float a[16], const float b[16])
{
  float          aBuf[32];
  float          bBuf[32];
  const float  *A;
  const float  *B;

  if (a == dest) { cpyMat16(aBuf, a); A = aBuf; } else A = a;
  if (b == dest) { cpyMat16(bBuf, b); B = bBuf; } else B = b;

  dest[ 0] = A[ 0]*B[ 0] + A[ 1]*B[ 4] + A[ 2]*B[ 8] + A[ 3]*B[12];
  dest[ 1] = A[ 0]*B[ 1] + A[ 1]*B[ 5] + A[ 2]*B[ 9] + A[ 3]*B[13];
  dest[ 2] = A[ 0]*B[ 2] + A[ 1]*B[ 6] + A[ 2]*B[10] + A[ 3]*B[14];
  dest[ 3] = A[ 0]*B[ 3] + A[ 1]*B[ 7] + A[ 2]*B[11] + A[ 3]*B[15];

  dest[ 4] = A[ 4]*B[ 0] + A[ 5]*B[ 4] + A[ 6]*B[ 8] + A[ 7]*B[12];
  dest[ 5] = A[ 4]*B[ 1] + A[ 5]*B[ 5] + A[ 6]*B[ 9] + A[ 7]*B[13];
  dest[ 6] = A[ 4]*B[ 2] + A[ 5]*B[ 6] + A[ 6]*B[10] + A[ 7]*B[14];
  dest[ 7] = A[ 4]*B[ 3] + A[ 5]*B[ 7] + A[ 6]*B[11] + A[ 7]*B[15];

  dest[ 8] = A[ 8]*B[ 0] + A[ 9]*B[ 4] + A[10]*B[ 8] + A[11]*B[12];
  dest[ 9] = A[ 8]*B[ 1] + A[ 9]*B[ 5] + A[10]*B[ 9] + A[11]*B[13];
  dest[10] = A[ 8]*B[ 2] + A[ 9]*B[ 6] + A[10]*B[10] + A[11]*B[14];
  dest[11] = A[ 8]*B[ 3] + A[ 9]*B[ 7] + A[10]*B[11] + A[11]*B[15];

  dest[12] = A[12]*B[ 0] + A[13]*B[ 4] + A[14]*B[ 8] + A[15]*B[12];
  dest[13] = A[12]*B[ 1] + A[13]*B[ 5] + A[14]*B[ 9] + A[15]*B[13];
  dest[14] = A[12]*B[ 2] + A[13]*B[ 6] + A[14]*B[10] + A[15]*B[14];
  dest[15] = A[12]*B[ 3] + A[13]*B[ 7] + A[14]*B[11] + A[15]*B[15];
}

void sflMat::mulMat16Mat16(double dest[16], const double a[16], const double b[16])
{
  double         aBuf[32];
  double         bBuf[32];
  const double  *A;
  const double  *B;

  if (a == dest) { cpyMat16(aBuf, a); A = aBuf; } else A = a;
  if (b == dest) { cpyMat16(bBuf, b); B = bBuf; } else B = b;

  dest[ 0] = A[ 0]*B[ 0] + A[ 1]*B[ 4] + A[ 2]*B[ 8] + A[ 3]*B[12];
  dest[ 1] = A[ 0]*B[ 1] + A[ 1]*B[ 5] + A[ 2]*B[ 9] + A[ 3]*B[13];
  dest[ 2] = A[ 0]*B[ 2] + A[ 1]*B[ 6] + A[ 2]*B[10] + A[ 3]*B[14];
  dest[ 3] = A[ 0]*B[ 3] + A[ 1]*B[ 7] + A[ 2]*B[11] + A[ 3]*B[15];

  dest[ 4] = A[ 4]*B[ 0] + A[ 5]*B[ 4] + A[ 6]*B[ 8] + A[ 7]*B[12];
  dest[ 5] = A[ 4]*B[ 1] + A[ 5]*B[ 5] + A[ 6]*B[ 9] + A[ 7]*B[13];
  dest[ 6] = A[ 4]*B[ 2] + A[ 5]*B[ 6] + A[ 6]*B[10] + A[ 7]*B[14];
  dest[ 7] = A[ 4]*B[ 3] + A[ 5]*B[ 7] + A[ 6]*B[11] + A[ 7]*B[15];

  dest[ 8] = A[ 8]*B[ 0] + A[ 9]*B[ 4] + A[10]*B[ 8] + A[11]*B[12];
  dest[ 9] = A[ 8]*B[ 1] + A[ 9]*B[ 5] + A[10]*B[ 9] + A[11]*B[13];
  dest[10] = A[ 8]*B[ 2] + A[ 9]*B[ 6] + A[10]*B[10] + A[11]*B[14];
  dest[11] = A[ 8]*B[ 3] + A[ 9]*B[ 7] + A[10]*B[11] + A[11]*B[15];

  dest[12] = A[12]*B[ 0] + A[13]*B[ 4] + A[14]*B[ 8] + A[15]*B[12];
  dest[13] = A[12]*B[ 1] + A[13]*B[ 5] + A[14]*B[ 9] + A[15]*B[13];
  dest[14] = A[12]*B[ 2] + A[13]*B[ 6] + A[14]*B[10] + A[15]*B[14];
  dest[15] = A[12]*B[ 3] + A[13]*B[ 7] + A[14]*B[11] + A[15]*B[15];
}

void  sflMat::mulMat32Mat32(double dest[32], const double a[32], const double b[32])
{
  mulMat16Mat16(dest, a, b);
  mulMat16Mat16(dest+16, b+16, a+16);
}

void  sflMat::mulMat32Inv32(double dest[32], const double a[32], const double bInv[32])
{
  mulMat16Mat16(dest, a, bInv+16);
  mulMat16Mat16(dest+16, bInv, a+16);
}

void  sflMat::cpyMat16(float dest[16], const float a[16])
{
  memcpy(dest, a, 16*sizeof(float));
}

void  sflMat::cpyMat16(float dest[16], const double a[16])
{
  for (int i=0; i<16; i++)
    dest[i] = (float)a[i];
}

void  sflMat::cpyMat16(double dest[16], const double a[16])
{
  memcpy(dest, a, 16*sizeof(double));
}

void  sflMat::cpyMat32(float dest[16], const float a[16])
{
  memcpy(dest, a, 32*sizeof(float));
}

void  sflMat::cpyMat32(double dest[16], const double a[16])
{
  memcpy(dest, a, 32*sizeof(double));
}

void  sflMat::cpyTrp16(float dest[16], const float a[16])
{
  for (int i=0; i<4; i++)
    {
      dest[4*i + 0] = a[ 0 + i];
      dest[4*i + 1] = a[ 4 + i];
      dest[4*i + 2] = a[ 8 + i];
      dest[4*i + 3] = a[12 + i];
    }
}

void  sflMat::cpyTrp16(double dest[16], const double a[16])
{
  for (int i=0; i<4; i++)
    {
      dest[4*i + 0] = a[ 0 + i];
      dest[4*i + 1] = a[ 4 + i];
      dest[4*i + 2] = a[ 8 + i];
      dest[4*i + 3] = a[12 + i];
    }
}

///////////////////////////////////////////////////////////////////////////////

void  sflMat::dumpMat32(FILE *out, const double mat[32])
{
  int  i;
  fprintf(stderr, "[\n");
  for (i=0; i<8; i++)
    {
      fprintf(stderr, "%6.4g %6.4g %6.4g %6.4g\n",
              mat[4*i + 0], mat[4*i + 1], mat[4*i + 2], mat[4*i + 3]);
      if (i==3)
        fprintf(stderr, "------------------------------------------------\n");
    }
  fprintf(stderr, "]\n");
}

void  sflMat::dumpMat16(FILE *out, const double mat[16])
{
  int  i;
  fprintf(stderr, "[\n");
  for (i=0; i<4; i++)
    {
      fprintf(stderr, "%6.4g %6.4g %6.4g %6.4g\n",
              mat[4*i + 0], mat[4*i + 1], mat[4*i + 2], mat[4*i + 3]);
    }
  fprintf(stderr, "]\n");
}

///////////////////////////////////////////////////////////////////////////////

void  sflMat::debugInversion(void)
{
  int  i, j;

  for (i=0; i<10000; i++)
    {
      double  m[32];
      double  id[16];
      for (j=0; j<16; j++)
        m[j] = 10.0*rand()/RAND_MAX;
      fprintf(stderr, ".");
      if (!updateInverse(m))
        fprintf(stderr, "SINGULAR MATRIX!\n");
      else
        {
          mulMat16Mat16(id, m, m+16);
          if (!isIdentity(id))
            {
              fprintf(stderr, "-------------------------------------------------------------------------------\n");
              dumpMat32(stderr, m);
              dumpMat16(stderr, id);
              fprintf(stderr, "-------------------------------------------------------------------------------\n");
            }
        }
    }
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
