/* ### CHECK-ME-BEFORE-RELEASE ### */
/*
 * Title:   normalenc.c
 * Created: Sun Apr 28 15:13:30 2002
 * Authors: Tim Weyrich <weyrich@inf.ethz.ch>
 *
 * Copyright (c) 2001--2003, Computer Graphics Lab, ETH Zurich
 *
 * This file is part of the Pointshop3D system.
 * See http://www.pointshop3d.com/ for more information.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 * Contact info@pointshop3d.com if any conditions of this
 * licensing are not clear to you.
 *
 * ---------------------------------------------------------------
 *
 * $Log: normalenc.c,v $
 * Revision 1.2  2003/11/19 13:04:02  weyrich
 * added Pointshop3D headers
 *
 * Revision 1.1.1.1  2002/05/08 08:58:47  weyrich
 * surfel file format
 */

#include <stdio.h>
#include <math.h>
#include <assert.h>

#include "normalenc.h"

/*****************************************************************************/

uint32  normalenc_encode_normal_ico8(double x, double y, double z)
{
  /* FIXME: not yet implemented */
  fprintf(stderr, "%s:%d: NOT YET IMPLEMENTED\n", __FILE__, __LINE__);
  return NORMALENC_CODE_INVALID;
}

/* Cubic, 16 Bit (6 bit per coord):
 *
 *  5432109876543210
 *  |-|0|----||----|
 *   |    |     |
 *   |    |     +- x or y
 *   |    +- y or z
 *   +- principal axis
 *
 */
#define  NORMALENC_CUB16_ZP    0x0000
#define  NORMALENC_CUB16_ZN    0x2000
#define  NORMALENC_CUB16_YP    0x4000
#define  NORMALENC_CUB16_YN    0x6000
#define  NORMALENC_CUB16_XP    0x8000
#define  NORMALENC_CUB16_XN    0xa000
#define  NORMALENC_CUB16_NL    0xc000

#define  NORMALENC_CUB16_MASK_0  0x003f
#define  NORMALENC_CUB16_MASK_1  0x0fc0
#define  NORMALENC_CUB16_MASK_P  0xe000

/*  normalenc_encode_normal_cubxx costs:
 *
 *    3 FABS
 *    4 FCMP
 *    1 FDIV
 *    2 FMUL
 *    2 FADD
 *    1  SHL
 *    2   OR
 */
uint32  normalenc_encode_normal_cub16(double x, double y, double z)
{
  if (fabs(z) > fabs(y))
    if (fabs(z) > fabs(x))
      { /* project on xy */
        if (fabs(z) < 1e-6)
          return NORMALENC_CUB16_NL;
        else
          {
            x *= (31.5/fabs(z));
            y *= (31.5/fabs(z));
            return (((z > 0) ? NORMALENC_CUB16_ZP : NORMALENC_CUB16_ZN) |
                    ((unsigned)(x+32)) |
                    (((unsigned)(y+32)) << 6));
          }
      }
    else
      { /* project on yz */
        if (fabs(x) < 1e-6)
          return NORMALENC_CUB16_NL;
        else
          {
            y *= (31.5/fabs(x));
            z *= (31.5/fabs(x));
            return (((x > 0) ? NORMALENC_CUB16_XP : NORMALENC_CUB16_XN) |
                    ((unsigned)(y+32)) |
                    (((unsigned)(z+32)) << 6));
          }
      }
  else
    if (fabs(y) > fabs(x))
      { /* project on xz */
        if (fabs(y) < 1e-6)
          return NORMALENC_CUB16_NL;
        else
          {
            x *= (31.5/fabs(y));
            z *= (31.5/fabs(y));
            return (((y > 0) ? NORMALENC_CUB16_YP : NORMALENC_CUB16_YN) |
                    ((unsigned)(x+32)) |
                    (((unsigned)(z+32)) << 6));
          }
      }
    else
      { /* project on yz */
        if (fabs(x) < 1e-6)
          return NORMALENC_CUB16_NL;
        else
          {
            y *= (31.5/fabs(x));
            z *= (31.5/fabs(x));
            return (((x > 0) ? NORMALENC_CUB16_XP : NORMALENC_CUB16_XN) |
                    ((unsigned)(y+32)) |
                    (((unsigned)(z+32)) << 6));
          }
      }
}

/* Cubic, 32 Bit (14 bit per coord):
 *
 *  10987654321098765432109876543210
 *  |-|0|------------||------------|
 *   |        |             |
 *   |        |             +- x or y
 *   |        +- y or z
 *   +- principal axis
 *
 */
#define  NORMALENC_CUB32_ZP    0x00000000UL
#define  NORMALENC_CUB32_ZN    0x20000000UL
#define  NORMALENC_CUB32_YP    0x40000000UL
#define  NORMALENC_CUB32_YN    0x60000000UL
#define  NORMALENC_CUB32_XP    0x80000000UL
#define  NORMALENC_CUB32_XN    0xa0000000UL
#define  NORMALENC_CUB32_NL    0xc0000000UL

#define  NORMALENC_CUB32_MASK_0  0x00003fffUL
#define  NORMALENC_CUB32_MASK_1  0x0fffc000UL
#define  NORMALENC_CUB32_MASK_P  0xe0000000UL

uint32  normalenc_encode_normal_cub32(double x, double y, double z)
{
  if (fabs(z) > fabs(y))
    if (fabs(z) > fabs(x))
      { /* project on xy */
        if (fabs(z) < 1e-12)
          return NORMALENC_CUB32_NL;
        else
          {
            x *= (8191.5/fabs(z));
            y *= (8191.5/fabs(z));
            return (((z > 0) ? NORMALENC_CUB32_ZP : NORMALENC_CUB32_ZN) |
                    ((unsigned)(x+8192)) |
                    (((unsigned)(y+8192)) << 14));
          }
      }
    else
      { /* project on yz */
        if (fabs(x) < 1e-12)
          return NORMALENC_CUB32_NL;
        else
          {
            y *= (8191.5/fabs(x));
            z *= (8191.5/fabs(x));
            return (((x > 0) ? NORMALENC_CUB32_XP : NORMALENC_CUB32_XN) |
                    ((unsigned)(y+8192)) |
                    (((unsigned)(z+8192)) << 14));
          }
      }
  else
    if (fabs(y) > fabs(x))
      { /* project on xz */
        if (fabs(y) < 1e-12)
          return NORMALENC_CUB32_NL;
        else
          {
            x *= (8191.5/fabs(y));
            z *= (8191.5/fabs(y));
            return (((y > 0) ? NORMALENC_CUB32_YP : NORMALENC_CUB32_YN) |
                    ((unsigned)(x+8192)) |
                    (((unsigned)(z+8192)) << 14));
          }
      }
    else
      { /* project on yz */
        if (fabs(x) < 1e-12)
          return NORMALENC_CUB32_NL;
        else
          {
            y *= (8191.5/fabs(x));
            z *= (8191.5/fabs(x));
            return (((x > 0) ? NORMALENC_CUB32_XP : NORMALENC_CUB32_XN) |
                    ((unsigned)(y+8192)) |
                    (((unsigned)(z+8192)) << 14));
          }
      }
}

/*****************************************************************************/

#define  NORMALENC_DECODE_NORMAL_ICO8(code,x,y,z)                       \
do {                                                                    \
  /* FIXME: not yet implemented */                                      \
  fprintf(stderr, "%s:%d: NOT YET IMPLEMENTED\n", __FILE__, __LINE__);  \
  return 1;                                                             \
} while(0)

int  normalenc_decode_normal_ico8_3f(uint32 code, float *x, float *y, float *z)
{
  NORMALENC_DECODE_NORMAL_ICO8(code, x, y, z);
}

int  normalenc_decode_normal_ico8_3d(uint32 code, double *x, double *y, double *z)
{
  NORMALENC_DECODE_NORMAL_ICO8(code, x, y, z);
}

/* normalenc_decode_normal_cubxx_xx() costs:
 *
 *   1 SQRT
 *   1 FDIV
 *   6 FMUL
 *   4 FADD
 *   2  AND
 *   1  SHR
 *   1 switch() of 7 CMPs (may be optimized by branch table)
 *
 */
#define  NORMALENC_DECODE_NORMAL_CUB16(code,x,y,z,type)                                 \
do {                                                                                    \
  double  u, v, f;                                                                      \
  u = (code & NORMALENC_CUB16_MASK_0) - 31.5;                                           \
  v = ((code & NORMALENC_CUB16_MASK_1) >> 6) - 31.5;                                    \
  f = 1.0 / sqrt((31.5*31.5) + u*u + v*v);                                              \
  u *= f;                                                                               \
  v *= f;                                                                               \
  assert(fabs(sqrt((31.5*31.5*f*f) + u*u + v*v) - 1.0) < 1e-6);                         \
  switch(code & NORMALENC_CUB16_MASK_P)                                                 \
    {                                                                                   \
    case NORMALENC_CUB16_ZP:  *x = (type)u; *y = (type)v; *z =  31.5f * (type)f; break; \
    case NORMALENC_CUB16_ZN:  *x = (type)u; *y = (type)v; *z = -31.5f * (type)f; break; \
    case NORMALENC_CUB16_YP:  *x = (type)u; *z = (type)v; *y =  31.5f * (type)f; break; \
    case NORMALENC_CUB16_YN:  *x = (type)u; *z = (type)v; *y = -31.5f * (type)f; break; \
    case NORMALENC_CUB16_XP:  *y = (type)u; *z = (type)v; *x =  31.5f * (type)f; break; \
    case NORMALENC_CUB16_XN:  *y = (type)u; *z = (type)v; *x = -31.5f * (type)f; break; \
    case NORMALENC_CUB16_NL:  *x = 0; *y = 0; *z = 0; break;                            \
    default:                                                                            \
      return 1;                                                                         \
    }                                                                                   \
  return 0;                                                                             \
} while(0)

int  normalenc_decode_normal_cub16_3f(uint32 code, float *x, float *y, float *z)
{
  NORMALENC_DECODE_NORMAL_CUB16(code, x, y, z, float);
}

int  normalenc_decode_normal_cub16_3d(uint32 code, double *x, double *y, double *z)
{
  NORMALENC_DECODE_NORMAL_CUB16(code, x, y, z, double);
}

#define  NORMALENC_DECODE_NORMAL_CUB32(code,x,y,z,type)                                         \
do {                                                                                            \
  double  u, v, f;                                                                              \
  u = (code & NORMALENC_CUB32_MASK_0) - 8191.5;                                                 \
  v = ((code & NORMALENC_CUB32_MASK_1) >> 14) - 8191.5;                                         \
  f = 1.0 / sqrt((8191.5*8191.5) + u*u + v*v);                                                  \
  u *= f;                                                                                       \
  v *= f;                                                                                       \
  switch(code & NORMALENC_CUB32_MASK_P)                                                         \
    {                                                                                           \
    case NORMALENC_CUB32_ZP:  *x = (type)u; *y = (type)v; *z =  8191.5f * (type)f; break;       \
    case NORMALENC_CUB32_ZN:  *x = (type)u; *y = (type)v; *z = -8191.5f * (type)f; break;       \
    case NORMALENC_CUB32_YP:  *x = (type)u; *z = (type)v; *y =  8191.5f * (type)f; break;       \
    case NORMALENC_CUB32_YN:  *x = (type)u; *z = (type)v; *y = -8191.5f * (type)f; break;       \
    case NORMALENC_CUB32_XP:  *y = (type)u; *z = (type)v; *x =  8191.5f * (type)f; break;       \
    case NORMALENC_CUB32_XN:  *y = (type)u; *z = (type)v; *x = -8191.5f * (type)f; break;       \
    case NORMALENC_CUB32_NL:  *x = 0; *y = 0; *z = 0; break;                                    \
    default:                                                                                    \
      return 1;                                                                                 \
    }                                                                                           \
  return 0;                                                                                     \
} while(0)

int  normalenc_decode_normal_cub32_3f(uint32 code, float *x, float *y, float *z)
{
  NORMALENC_DECODE_NORMAL_CUB32(code, x, y, z, float);
}

int  normalenc_decode_normal_cub32_3d(uint32 code, double *x, double *y, double *z)
{
  NORMALENC_DECODE_NORMAL_CUB32(code, x, y, z, double);
}

/*****************************************************************************/

uint32  normalenc_encode_normal(normalenc_compression_enum mode,
                                double x, double y, double z)
{
  switch (mode)
    {
    case NONE:           break;
    case ICOSAHEDRAL_8:  return normalenc_encode_normal_ico8(x, y, z);
    case CUBIC_16:       return normalenc_encode_normal_cub16(x, y, z);
    case CUBIC_32:       return normalenc_encode_normal_cub32(x, y, z);
    }
  return NORMALENC_CODE_INVALID;
}

int  normalenc_decode_normal_3f(normalenc_compression_enum mode,
                                uint32 code, float *x, float *y, float *z)
{
  switch (mode)
    {
    case NONE:           break;
    case ICOSAHEDRAL_8:  return normalenc_decode_normal_ico8_3f(code, x, y, z);
    case CUBIC_16:       return normalenc_decode_normal_cub16_3f(code, x, y, z);
    case CUBIC_32:       return normalenc_decode_normal_cub32_3f(code, x, y, z);
    }
  return 1;
}

int  normalenc_decode_normal_3d(normalenc_compression_enum mode,
                                uint32 code, double *x, double *y, double *z)
{
  switch (mode)
    {
    case NONE:           break;
    case ICOSAHEDRAL_8:  return normalenc_decode_normal_ico8_3d(code, x, y, z);
    case CUBIC_16:       return normalenc_decode_normal_cub16_3d(code, x, y, z);
    case CUBIC_32:       return normalenc_decode_normal_cub32_3d(code, x, y, z);
    }
  return 1;
}

/*****************************************************************************/

/* Some Emacs-Hints -- please don't remove:
 *
 *  Local Variables:
 *  tab-width:4
 *  End:
 */
