// ### CHECK-ME-BEFORE-RELEASE ###
//
// Title:   testmain.cpp
// Created: Tue Apr 16 14:14:30 2002
// Authors: Tim Weyrich <weyrich@inf.ethz.ch>
//
// Copyright (c) 2001--2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//
// ---------------------------------------------------------------
//
// $Log: testmain.cpp,v $
// Revision 1.2  2003/11/19 13:04:03  weyrich
// added Pointshop3D headers
//
// Revision 1.1.1.1  2002/05/08 08:58:47  weyrich
// surfel file format
//

#include <pisystem.h>
#include <errno.h>
#include <pistring.h>

#include <assert.h>

#include <sfl.h>

int  read_test(void)
{
  fprintf(stderr, "READING...\n");
  {
    gff_file  *in = gff_open("test.gff", "r", SFL_MAGIC, SFL_VERSION);

    {
      gff_directory  *dir = gff_read_root_dir(in);
      assert(dir);
      fprintf(stderr, "Num children: %08x\n", (unsigned)gff_dir_get_num_children(dir));

#if 1
      gff_read_next_dir(in, dir);

      fprintf(stderr, "Num children: %08x\n", (unsigned)gff_dir_get_num_children(dir));
#endif
    }
    if (gff_close(in))
      {
        fprintf(stderr, "%s%d: error on close: %s\n", __FILE__, __LINE__, strerror(errno));
        return 1;
      }
  }
  return 0;
}

int  write_test(void)
{
  fprintf(stderr, "WRITING...\n");
  {
    gff_file  *out = gff_open("test.gff", "w", SFL_MAGIC, SFL_VERSION);

#if 0
    gff_assume_not_seekable(out);
#endif

    {
      gff_write_field(out, SFLTAG_VERSION, GFFTYPE_UINT32, 1, 0x12345678);
      gff_write_field_cstring(out, SFLTAG_TITLE, "GFF Test");

#if 0
      gff_write_downlink(out, GFF_UNKNOWN_SIZE);
#else
      gff_write_downlink(out, 2);
#endif
      {
        gff_write_field_cstring(out, SFLTAG_TITLE, "Text in child node!");
        gff_break_dir(out);
        gff_write_field_cstring(out, SFLTAG_TITLE, "Another text in a child node!");
        gff_write_field_cstring(out, SFLTAG_AUTHOR, "Now, some data will follow...");
        gff_write_data_field(out, SFLTAG_SURFEL_SET, 317);
        gff_write(out, malloc(320), 317);
        gff_write_data_field(out, SFLTAG_SURFEL_SET, 317);
        gff_write(out, malloc(320), 317);
      }
      gff_write_uplink(out);
    }

    if (gff_close(out))
      {
        fprintf(stderr, "%s%d: error on close: %s\n", __FILE__, __LINE__, strerror(errno));
        return 1;
      }
  }
  return 0;
}

int  main(void)
{
  return write_test() || read_test();
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
